/*****************************************************************************
 *
 * grail - Gesture Recognition And Instantiation Library
 *
 * Copyright (C) 2010 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Henrik Rydberg <rydberg@bitmath.org>
 *
 ****************************************************************************/

#include <touch-engine.h>
#include <string.h>

static void tp_event(struct touch_dev *dev, const struct input_event *ev)
{
}

static void update_frame(struct touch_frame *frame,
			 int slot, int active,
			 const touch_prop_mask_t *mask,
			 const touch_prop_t *prop)
{
	struct touch *touch = &frame->touch[slot];
	touch->active = active;
	if (mask && prop) {
		for (int i = 0; i < DIM_TOUCH_PROP; i++)
			if (has_prop(mask, i))
				touch->prop[i] = prop[i];
	}
}

static void finalize_frame(struct touch_frame *frame)
{
	int last = -1;
	frame->ntouch = 0;
	frame->slot = -1;
	for (int i = 0; i < DIM_TOUCH_PROP; i++) {
		frame->touch[i].next = -1;
		if (frame->touch[i].active) {
			if (last < 0)
				frame->slot = i;
			else
				frame->touch[last].next = i;
			last = i;
			frame->ntouch++;
		}
	}
}

static void tp_create(struct touch_dev *dev, int slot,
		      const touch_prop_mask_t *mask, const touch_prop_t *prop)
{
	struct touch_engine *engine = dev->priv;
	if (engine)
		update_frame(&engine->frame, slot, 1, mask, prop);
}

static void tp_modify(struct touch_dev *dev, int slot,
		      const touch_prop_mask_t *mask, const touch_prop_t *prop)
{
	struct touch_engine *engine = dev->priv;
	if (engine)
		update_frame(&engine->frame, slot, 1, mask, prop);
}

static void tp_destroy(struct touch_dev *dev, int slot)
{
	struct touch_engine *engine = dev->priv;
	if (engine)
		update_frame(&engine->frame, slot, 0, 0, 0);
}

static void tp_sync(struct touch_dev *dev, touch_time_t time)
{
	struct touch_engine *engine = dev->priv;
	if (engine) {
		finalize_frame(&engine->frame);
		if (engine->sync)
			engine->sync(engine, time);
	}
}

void touch_engine_init(struct touch_engine *engine,
		       void (*sync)(struct touch_engine *engine,
				    touch_time_t time),
		       void *priv)
{
	memset(engine, 0, sizeof(*engine));
	engine->sync = sync;
	engine->priv = priv;
}

void touch_engine_attach(struct touch_engine *engine, struct touch_dev *dev)
{
	dev->priv = engine;
	dev->event = tp_event;
	dev->create = tp_create;
	dev->modify = tp_modify;
	dev->destroy = tp_destroy;
	dev->sync = tp_sync;
}

void touch_engine_detach(struct touch_engine *engine, struct touch_dev *dev)
{
	dev->sync = 0;
	dev->destroy = 0;
	dev->modify = 0;
	dev->create = 0;
	dev->event = 0;
	dev->priv = 0;
}

