/*****************************************************************************
 *
 * grail - Gesture Recognition And Instantiation Library
 *
 * Copyright (C) 2010 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Henrik Rydberg <rydberg@bitmath.org>
 *
 ****************************************************************************/

#include <grail.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

static void tp_sync(struct touch_engine *engine, touch_time_t time)
{
	struct touch_frame *frame = &engine->frame;
	int slot, i;
	fprintf(stderr, "sync %d %lld\n", frame->ntouch, time);
	for (slot = frame->slot; slot >= 0; slot = next_slot(frame, slot)) {
		struct touch *touch = &frame->touch[slot];
		fprintf(stderr, "touch %d\n", slot);
		for (i = 0; i < DIM_TOUCH_PROP; i++)
			fprintf(stderr, "  %d\n", touch->prop[i]);
	}
}

static void loop_device(struct touch_dev *dev, int fd)
{
	struct touch_engine engine;
	touch_engine_init(&engine, tp_sync, 0);
	touch_engine_attach(&engine, dev);
	while (!touch_dev_idle(dev, fd, 5000))
		touch_dev_pull(dev, fd);
	touch_engine_detach(&engine, dev);
}

int main(int argc, char *argv[])
{
	struct touch_dev dev;
	int fd;
	if (argc < 2) {
		fprintf(stderr, "Usage: mtdev <device>\n");
		return -1;
	}
	fd = open(argv[1], O_RDONLY | O_NONBLOCK);
	if (fd < 0) {
		fprintf(stderr, "error: could not open device\n");
		return -1;
	}
	if (ioctl(fd, EVIOCGRAB, 1)) {
		fprintf(stderr, "error: could not grab the device\n");
		return -1;
	}
	if (touch_dev_open(&dev, fd)) {
		fprintf(stderr, "error: could not open touch device\n");
		return -1;
	}
	loop_device(&dev, fd);
	touch_dev_close(&dev, fd);
	ioctl(fd, EVIOCGRAB, 0);
	close(fd);
	return 0;
}
